@@ -2,13 +2,16 @@
 
   math.c -
 
-  $Author: akr $
+  $Author: nobu $
   created at: Tue Jan 25 14:12:56 JST 1994
 
   Copyright (C) 1993-2007 Yukihiro Matsumoto
 
 **********************************************************************/
 
+#ifdef _MSC_VER
+# define _USE_MATH_DEFINES 1
+#endif
 #include "internal.h"
 #include <float.h>
 #include <math.h>
@@ -24,11 +27,7 @@
 VALUE rb_mMath;
 VALUE rb_eMathDomainError;
 
-#define Need_Float(x) do {if (!RB_TYPE_P(x, T_FLOAT)) {(x) = rb_to_float(x);}} while(0)
-#define Need_Float2(x,y) do {\
-    Need_Float(x);\
-    Need_Float(y);\
-} while (0)
+#define Get_Double(x) rb_num_to_dbl(x)
 
 #define domain_error(msg) \
     rb_raise(rb_eMathDomainError, "Numerical argument is out of domain - " #msg)
@@ -38,7 +37,9 @@ VALUE rb_eMathDomainError;
  *     Math.atan2(y, x)  -> Float
  *
  *  Computes the arc tangent given +y+ and +x+.
- *  Returns a Float in the range -PI..PI.
+ *  Returns a Float in the range -PI..PI. Return value is a angle
+ *  in radians between the positive x-axis of cartesian plane
+ *  and the point given by the coordinates (+x+, +y+) on it.
  *
  *  Domain: (-INFINITY, INFINITY)
  *
@@ -62,15 +63,11 @@ VALUE rb_eMathDomainError;
  */
 
 static VALUE
-math_atan2(VALUE obj, VALUE y, VALUE x)
+math_atan2(VALUE unused_obj, VALUE y, VALUE x)
 {
-#ifndef M_PI
-# define M_PI 3.14159265358979323846
-#endif
     double dx, dy;
-    Need_Float2(y, x);
-    dx = RFLOAT_VALUE(x);
-    dy = RFLOAT_VALUE(y);
+    dx = Get_Double(x);
+    dy = Get_Double(y);
     if (dx == 0.0 && dy == 0.0) {
 	if (!signbit(dx))
 	    return DBL2NUM(dy);
@@ -111,10 +108,9 @@ math_atan2(VALUE obj, VALUE y, VALUE x)
  */
 
 static VALUE
-math_cos(VALUE obj, VALUE x)
+math_cos(VALUE unused_obj, VALUE x)
 {
-    Need_Float(x);
-    return DBL2NUM(cos(RFLOAT_VALUE(x)));
+    return DBL2NUM(cos(Get_Double(x)));
 }
 
 /*
@@ -133,10 +129,9 @@ math_cos(VALUE obj, VALUE x)
  */
 
 static VALUE
-math_sin(VALUE obj, VALUE x)
+math_sin(VALUE unused_obj, VALUE x)
 {
-    Need_Float(x);
-    return DBL2NUM(sin(RFLOAT_VALUE(x)));
+    return DBL2NUM(sin(Get_Double(x)));
 }
 
 
@@ -155,10 +150,9 @@ math_sin(VALUE obj, VALUE x)
  */
 
 static VALUE
-math_tan(VALUE obj, VALUE x)
+math_tan(VALUE unused_obj, VALUE x)
 {
-    Need_Float(x);
-    return DBL2NUM(tan(RFLOAT_VALUE(x)));
+    return DBL2NUM(tan(Get_Double(x)));
 }
 
 /*
@@ -176,16 +170,14 @@ math_tan(VALUE obj, VALUE x)
  */
 
 static VALUE
-math_acos(VALUE obj, VALUE x)
+math_acos(VALUE unused_obj, VALUE x)
 {
-    double d0, d;
+    double d;
 
-    Need_Float(x);
-    d0 = RFLOAT_VALUE(x);
+    d = Get_Double(x);
     /* check for domain error */
-    if (d0 < -1.0 || 1.0 < d0) domain_error("acos");
-    d = acos(d0);
-    return DBL2NUM(d);
+    if (d < -1.0 || 1.0 < d) domain_error("acos");
+    return DBL2NUM(acos(d));
 }
 
 /*
@@ -202,16 +194,14 @@ math_acos(VALUE obj, VALUE x)
  */
 
 static VALUE
-math_asin(VALUE obj, VALUE x)
+math_asin(VALUE unused_obj, VALUE x)
 {
-    double d0, d;
+    double d;
 
-    Need_Float(x);
-    d0 = RFLOAT_VALUE(x);
+    d = Get_Double(x);
     /* check for domain error */
-    if (d0 < -1.0 || 1.0 < d0) domain_error("asin");
-    d = asin(d0);
-    return DBL2NUM(d);
+    if (d < -1.0 || 1.0 < d) domain_error("asin");
+    return DBL2NUM(asin(d));
 }
 
 /*
@@ -228,10 +218,9 @@ math_asin(VALUE obj, VALUE x)
  */
 
 static VALUE
-math_atan(VALUE obj, VALUE x)
+math_atan(VALUE unused_obj, VALUE x)
 {
-    Need_Float(x);
-    return DBL2NUM(atan(RFLOAT_VALUE(x)));
+    return DBL2NUM(atan(Get_Double(x)));
 }
 
 #ifndef HAVE_COSH
@@ -257,10 +246,9 @@ cosh(double x)
  */
 
 static VALUE
-math_cosh(VALUE obj, VALUE x)
+math_cosh(VALUE unused_obj, VALUE x)
 {
-    Need_Float(x);
-    return DBL2NUM(cosh(RFLOAT_VALUE(x)));
+    return DBL2NUM(cosh(Get_Double(x)));
 }
 
 #ifndef HAVE_SINH
@@ -286,17 +274,23 @@ sinh(double x)
  */
 
 static VALUE
-math_sinh(VALUE obj, VALUE x)
+math_sinh(VALUE unused_obj, VALUE x)
 {
-    Need_Float(x);
-    return DBL2NUM(sinh(RFLOAT_VALUE(x)));
+    return DBL2NUM(sinh(Get_Double(x)));
 }
 
 #ifndef HAVE_TANH
 double
 tanh(double x)
 {
-    return sinh(x) / cosh(x);
+# if defined(HAVE_SINH) && defined(HAVE_COSH)
+    const double c = cosh(x);
+    if (!isinf(c)) return sinh(x) / c;
+# else
+    const double e = exp(x+x);
+    if (!isinf(e)) return (e - 1) / (e + 1);
+# endif
+    return x > 0 ? 1.0 : -1.0;
 }
 #endif
 
@@ -315,10 +309,9 @@ tanh(double x)
  */
 
 static VALUE
-math_tanh(VALUE obj, VALUE x)
+math_tanh(VALUE unused_obj, VALUE x)
 {
-    Need_Float(x);
-    return DBL2NUM(tanh(RFLOAT_VALUE(x)));
+    return DBL2NUM(tanh(Get_Double(x)));
 }
 
 /*
@@ -336,16 +329,14 @@ math_tanh(VALUE obj, VALUE x)
  */
 
 static VALUE
-math_acosh(VALUE obj, VALUE x)
+math_acosh(VALUE unused_obj, VALUE x)
 {
-    double d0, d;
+    double d;
 
-    Need_Float(x);
-    d0 = RFLOAT_VALUE(x);
+    d = Get_Double(x);
     /* check for domain error */
-    if (d0 < 1.0) domain_error("acosh");
-    d = acosh(d0);
-    return DBL2NUM(d);
+    if (d < 1.0) domain_error("acosh");
+    return DBL2NUM(acosh(d));
 }
 
 /*
@@ -363,10 +354,9 @@ math_acosh(VALUE obj, VALUE x)
  */
 
 static VALUE
-math_asinh(VALUE obj, VALUE x)
+math_asinh(VALUE unused_obj, VALUE x)
 {
-    Need_Float(x);
-    return DBL2NUM(asinh(RFLOAT_VALUE(x)));
+    return DBL2NUM(asinh(Get_Double(x)));
 }
 
 /*
@@ -384,19 +374,17 @@ math_asinh(VALUE obj, VALUE x)
  */
 
 static VALUE
-math_atanh(VALUE obj, VALUE x)
+math_atanh(VALUE unused_obj, VALUE x)
 {
-    double d0, d;
+    double d;
 
-    Need_Float(x);
-    d0 = RFLOAT_VALUE(x);
+    d = Get_Double(x);
     /* check for domain error */
-    if (d0 <  -1.0 || +1.0 <  d0) domain_error("atanh");
+    if (d <  -1.0 || +1.0 <  d) domain_error("atanh");
     /* check for pole error */
-    if (d0 == -1.0) return DBL2NUM(-INFINITY);
-    if (d0 == +1.0) return DBL2NUM(+INFINITY);
-    d = atanh(d0);
-    return DBL2NUM(d);
+    if (d == -1.0) return DBL2NUM(-HUGE_VAL);
+    if (d == +1.0) return DBL2NUM(+HUGE_VAL);
+    return DBL2NUM(atanh(d));
 }
 
 /*
@@ -416,10 +404,9 @@ math_atanh(VALUE obj, VALUE x)
  */
 
 static VALUE
-math_exp(VALUE obj, VALUE x)
+math_exp(VALUE unused_obj, VALUE x)
 {
-    Need_Float(x);
-    return DBL2NUM(exp(RFLOAT_VALUE(x)));
+    return DBL2NUM(exp(Get_Double(x)));
 }
 
 #if defined __CYGWIN__
@@ -431,7 +418,15 @@ math_exp(VALUE obj, VALUE x)
 # define log10(x) ((x) < 0.0 ? nan("") : log10(x))
 #endif
 
+#ifndef M_LN2
+# define M_LN2 0.693147180559945309417232121458176568
+#endif
+#ifndef M_LN10
+# define M_LN10 2.30258509299404568401799145468436421
+#endif
+
 static double math_log1(VALUE x);
+FUNC_MINIMIZED(static VALUE math_log(int, const VALUE *, VALUE));
 
 /*
  *  call-seq:
@@ -455,7 +450,13 @@ static double math_log1(VALUE x);
  */
 
 static VALUE
-math_log(int argc, const VALUE *argv, VALUE obj)
+math_log(int argc, const VALUE *argv, VALUE unused_obj)
+{
+    return rb_math_log(argc, argv);
+}
+
+VALUE
+rb_math_log(int argc, const VALUE *argv)
 {
     VALUE x, base;
     double d;
@@ -469,9 +470,8 @@ math_log(int argc, const VALUE *argv, VALUE obj)
 }
 
 static double
-math_log1(VALUE x)
+get_double_rshift(VALUE x, size_t *pnumbits)
 {
-    double d0, d;
     size_t numbits;
 
     if (RB_BIGNUM_TYPE_P(x) && BIGNUM_POSITIVE_P(x) &&
@@ -482,17 +482,22 @@ math_log1(VALUE x)
     else {
 	numbits = 0;
     }
+    *pnumbits = numbits;
+    return Get_Double(x);
+}
+
+static double
+math_log1(VALUE x)
+{
+    size_t numbits;
+    double d = get_double_rshift(x, &numbits);
 
-    Need_Float(x);
-    d0 = RFLOAT_VALUE(x);
     /* check for domain error */
-    if (d0 < 0.0) domain_error("log");
+    if (d < 0.0) domain_error("log");
     /* check for pole error */
-    if (d0 == 0.0) return -INFINITY;
-    d = log(d0);
-    if (numbits)
-        d += numbits * log(2); /* log(2**numbits) */
-    return d;
+    if (d == 0.0) return -HUGE_VAL;
+
+    return log(d) + numbits * M_LN2; /* log(d * 2 ** numbits) */
 }
 
 #ifndef log2
@@ -525,29 +530,17 @@ extern double log2(double);
  */
 
 static VALUE
-math_log2(VALUE obj, VALUE x)
+math_log2(VALUE unused_obj, VALUE x)
 {
-    double d0, d;
     size_t numbits;
+    double d = get_double_rshift(x, &numbits);
 
-    if (RB_BIGNUM_TYPE_P(x) && BIGNUM_POSITIVE_P(x) &&
-            DBL_MAX_EXP <= (numbits = rb_absint_numwords(x, 1, NULL))) {
-        numbits -= DBL_MANT_DIG;
-        x = rb_big_rshift(x, SIZET2NUM(numbits));
-    }
-    else {
-	numbits = 0;
-    }
-
-    Need_Float(x);
-    d0 = RFLOAT_VALUE(x);
     /* check for domain error */
-    if (d0 < 0.0) domain_error("log2");
+    if (d < 0.0) domain_error("log2");
     /* check for pole error */
-    if (d0 == 0.0) return DBL2NUM(-INFINITY);
-    d = log2(d0);
-    d += numbits;
-    return DBL2NUM(d);
+    if (d == 0.0) return DBL2NUM(-HUGE_VAL);
+
+    return DBL2NUM(log2(d) + numbits); /* log2(d * 2 ** numbits) */
 }
 
 /*
@@ -567,30 +560,17 @@ math_log2(VALUE obj, VALUE x)
  */
 
 static VALUE
-math_log10(VALUE obj, VALUE x)
+math_log10(VALUE unused_obj, VALUE x)
 {
-    double d0, d;
     size_t numbits;
+    double d = get_double_rshift(x, &numbits);
 
-    if (RB_BIGNUM_TYPE_P(x) && BIGNUM_POSITIVE_P(x) &&
-            DBL_MAX_EXP <= (numbits = rb_absint_numwords(x, 1, NULL))) {
-        numbits -= DBL_MANT_DIG;
-        x = rb_big_rshift(x, SIZET2NUM(numbits));
-    }
-    else {
-	numbits = 0;
-    }
-
-    Need_Float(x);
-    d0 = RFLOAT_VALUE(x);
     /* check for domain error */
-    if (d0 < 0.0) domain_error("log10");
+    if (d < 0.0) domain_error("log10");
     /* check for pole error */
-    if (d0 == 0.0) return DBL2NUM(-INFINITY);
-    d = log10(d0);
-    if (numbits)
-        d += numbits * log10(2); /* log10(2**numbits) */
-    return DBL2NUM(d);
+    if (d == 0.0) return DBL2NUM(-HUGE_VAL);
+
+    return DBL2NUM(log10(d) + numbits * log10(2)); /* log10(d * 2 ** numbits) */
 }
 
 /*
@@ -617,20 +597,58 @@ math_log10(VALUE obj, VALUE x)
  *    #   [8, 2.82842712474619, 8.0]
  *    #   [9, 3.0, 9.0]
  *    #   [10, 3.16227766016838, 10.0]
+ *
+ *  Note that the limited precision of floating point arithmetic
+ *  might lead to surprising results:
+ *
+ *    Math.sqrt(10**46).to_i  #=> 99999999999999991611392 (!)
+ *
+ *  See also BigDecimal#sqrt and Integer.sqrt.
  */
 
 static VALUE
-math_sqrt(VALUE obj, VALUE x)
+math_sqrt(VALUE unused_obj, VALUE x)
 {
-    double d0, d;
+    return rb_math_sqrt(x);
+}
 
-    Need_Float(x);
-    d0 = RFLOAT_VALUE(x);
+#define f_boolcast(x) ((x) ? Qtrue : Qfalse)
+inline static VALUE
+f_negative_p(VALUE x)
+{
+    if (FIXNUM_P(x))
+        return f_boolcast(FIX2LONG(x) < 0);
+    return rb_funcall(x, '<', 1, INT2FIX(0));
+}
+inline static VALUE
+f_signbit(VALUE x)
+{
+    if (RB_TYPE_P(x, T_FLOAT)) {
+        double f = RFLOAT_VALUE(x);
+        return f_boolcast(!isnan(f) && signbit(f));
+    }
+    return f_negative_p(x);
+}
+
+VALUE
+rb_math_sqrt(VALUE x)
+{
+    double d;
+
+    if (RB_TYPE_P(x, T_COMPLEX)) {
+	VALUE neg = f_signbit(RCOMPLEX(x)->imag);
+	double re = Get_Double(RCOMPLEX(x)->real), im;
+	d = Get_Double(rb_complex_abs(x));
+	im = sqrt((d - re) / 2.0);
+	re = sqrt((d + re) / 2.0);
+	if (neg) im = -im;
+	return rb_complex_new(DBL2NUM(re), DBL2NUM(im));
+    }
+    d = Get_Double(x);
     /* check for domain error */
-    if (d0 < 0.0) domain_error("sqrt");
-    if (d0 == 0.0) return DBL2NUM(0.0);
-    d = sqrt(d0);
-    return DBL2NUM(d);
+    if (d < 0.0) domain_error("sqrt");
+    if (d == 0.0) return DBL2NUM(0.0);
+    return DBL2NUM(sqrt(d));
 }
 
 /*
@@ -639,9 +657,9 @@ math_sqrt(VALUE obj, VALUE x)
  *
  *  Returns the cube root of +x+.
  *
- *  Domain: [0, INFINITY)
+ *  Domain: (-INFINITY, INFINITY)
  *
- *  Codomain:[0, INFINITY)
+ *  Codomain: (-INFINITY, INFINITY)
  *
  *    -9.upto(9) {|x|
  *      p [x, Math.cbrt(x), Math.cbrt(x)**3]
@@ -669,10 +687,16 @@ math_sqrt(VALUE obj, VALUE x)
  */
 
 static VALUE
-math_cbrt(VALUE obj, VALUE x)
+math_cbrt(VALUE unused_obj, VALUE x)
 {
-    Need_Float(x);
-    return DBL2NUM(cbrt(RFLOAT_VALUE(x)));
+    double f = Get_Double(x);
+    double r = cbrt(f);
+#if defined __GLIBC__
+    if (isfinite(r)) {
+	r = (2.0 * r + (f / r / r)) / 3.0;
+    }
+#endif
+    return DBL2NUM(r);
 }
 
 /*
@@ -680,21 +704,19 @@ math_cbrt(VALUE obj, VALUE x)
  *     Math.frexp(x)    -> [fraction, exponent]
  *
  *  Returns a two-element array containing the normalized fraction (a Float)
- *  and exponent (a Fixnum) of +x+.
+ *  and exponent (an Integer) of +x+.
  *
  *     fraction, exponent = Math.frexp(1234)   #=> [0.6025390625, 11]
  *     fraction * 2**exponent                  #=> 1234.0
  */
 
 static VALUE
-math_frexp(VALUE obj, VALUE x)
+math_frexp(VALUE unused_obj, VALUE x)
 {
     double d;
     int exp;
 
-    Need_Float(x);
-
-    d = frexp(RFLOAT_VALUE(x), &exp);
+    d = frexp(Get_Double(x), &exp);
     return rb_assoc_new(DBL2NUM(d), INT2NUM(exp));
 }
 
@@ -709,10 +731,9 @@ math_frexp(VALUE obj, VALUE x)
  */
 
 static VALUE
-math_ldexp(VALUE obj, VALUE x, VALUE n)
+math_ldexp(VALUE unused_obj, VALUE x, VALUE n)
 {
-    Need_Float(x);
-    return DBL2NUM(ldexp(RFLOAT_VALUE(x), NUM2INT(n)));
+    return DBL2NUM(ldexp(Get_Double(x), NUM2INT(n)));
 }
 
 /*
@@ -726,10 +747,9 @@ math_ldexp(VALUE obj, VALUE x, VALUE n)
  */
 
 static VALUE
-math_hypot(VALUE obj, VALUE x, VALUE y)
+math_hypot(VALUE unused_obj, VALUE x, VALUE y)
 {
-    Need_Float2(x, y);
-    return DBL2NUM(hypot(RFLOAT_VALUE(x), RFLOAT_VALUE(y)));
+    return DBL2NUM(hypot(Get_Double(x), Get_Double(y)));
 }
 
 /*
@@ -747,10 +767,9 @@ math_hypot(VALUE obj, VALUE x, VALUE y)
  */
 
 static VALUE
-math_erf(VALUE obj, VALUE x)
+math_erf(VALUE unused_obj, VALUE x)
 {
-    Need_Float(x);
-    return DBL2NUM(erf(RFLOAT_VALUE(x)));
+    return DBL2NUM(erf(Get_Double(x)));
 }
 
 /*
@@ -768,10 +787,9 @@ math_erf(VALUE obj, VALUE x)
  */
 
 static VALUE
-math_erfc(VALUE obj, VALUE x)
+math_erfc(VALUE unused_obj, VALUE x)
 {
-    Need_Float(x);
-    return DBL2NUM(erfc(RFLOAT_VALUE(x)));
+    return DBL2NUM(erfc(Get_Double(x)));
 }
 
 /*
@@ -815,7 +833,7 @@ math_erfc(VALUE obj, VALUE x)
  */
 
 static VALUE
-math_gamma(VALUE obj, VALUE x)
+math_gamma(VALUE unused_obj, VALUE x)
 {
     static const double fact_table[] = {
         /* fact(0) */ 1.0,
@@ -845,22 +863,24 @@ math_gamma(VALUE obj, VALUE x)
          * impossible to represent exactly in IEEE 754 double which have
          * 53bit mantissa. */
     };
-    double d0, d;
-    double intpart, fracpart;
-    Need_Float(x);
-    d0 = RFLOAT_VALUE(x);
+    enum {NFACT_TABLE = numberof(fact_table)};
+    double d;
+    d = Get_Double(x);
     /* check for domain error */
-    if (isinf(d0) && signbit(d0)) domain_error("gamma");
-    fracpart = modf(d0, &intpart);
-    if (fracpart == 0.0) {
-	if (intpart < 0) domain_error("gamma");
-	if (0 < intpart &&
-	    intpart - 1 < (double)numberof(fact_table)) {
-	    return DBL2NUM(fact_table[(int)intpart - 1]);
+    if (isinf(d)) {
+	if (signbit(d)) domain_error("gamma");
+	return DBL2NUM(HUGE_VAL);
+    }
+    if (d == 0.0) {
+	return signbit(d) ? DBL2NUM(-HUGE_VAL) : DBL2NUM(HUGE_VAL);
+    }
+    if (d == floor(d)) {
+	if (d < 0.0) domain_error("gamma");
+	if (1.0 <= d && d <= (double)NFACT_TABLE) {
+	    return DBL2NUM(fact_table[(int)d - 1]);
 	}
     }
-    d = tgamma(d0);
-    return DBL2NUM(d);
+    return DBL2NUM(tgamma(d));
 }
 
 /*
@@ -878,20 +898,22 @@ math_gamma(VALUE obj, VALUE x)
  */
 
 static VALUE
-math_lgamma(VALUE obj, VALUE x)
+math_lgamma(VALUE unused_obj, VALUE x)
 {
-    double d0, d;
+    double d;
     int sign=1;
     VALUE v;
-    Need_Float(x);
-    d0 = RFLOAT_VALUE(x);
+    d = Get_Double(x);
     /* check for domain error */
-    if (isinf(d0)) {
-	if (signbit(d0)) domain_error("lgamma");
-	return rb_assoc_new(DBL2NUM(INFINITY), INT2FIX(1));
+    if (isinf(d)) {
+	if (signbit(d)) domain_error("lgamma");
+	return rb_assoc_new(DBL2NUM(HUGE_VAL), INT2FIX(1));
     }
-    d = lgamma_r(d0, &sign);
-    v = DBL2NUM(d);
+    if (d == 0.0) {
+	VALUE vsign = signbit(d) ? INT2FIX(-1) : INT2FIX(+1);
+	return rb_assoc_new(DBL2NUM(HUGE_VAL), vsign);
+    }
+    v = DBL2NUM(lgamma_r(d, &sign));
     return rb_assoc_new(v, INT2FIX(sign));
 }
 
@@ -900,14 +922,14 @@ math_lgamma(VALUE obj, VALUE x)
 VALUE \
 rb_math_##n(VALUE x)\
 {\
-    return math_##n(rb_mMath, x);\
+    return math_##n(0, x);\
 }
 
 #define exp2(n) \
 VALUE \
 rb_math_##n(VALUE x, VALUE y)\
 {\
-    return math_##n(rb_mMath, x, y);\
+    return math_##n(0, x, y);\
 }
 
 exp2(atan2)
@@ -915,13 +937,6 @@ exp1(cos)
 exp1(cosh)
 exp1(exp)
 exp2(hypot)
-
-VALUE
-rb_math_log(int argc, const VALUE *argv)
-{
-    return math_log(argc, argv, rb_mMath);
-}
-
 exp1(sin)
 exp1(sinh)
 #if 0
@@ -958,17 +973,13 @@ exp1(sqrt)
 
 
 void
-Init_Math(void)
+InitVM_Math(void)
 {
     rb_mMath = rb_define_module("Math");
     rb_eMathDomainError = rb_define_class_under(rb_mMath, "DomainError", rb_eStandardError);
 
-#ifdef M_PI
     /*  Definition of the mathematical constant PI as a Float number. */
     rb_define_const(rb_mMath, "PI", DBL2NUM(M_PI));
-#else
-    rb_define_const(rb_mMath, "PI", DBL2NUM(atan(1.0)*4.0));
-#endif
 
 #ifdef M_E
     /*  Definition of the mathematical constant E (e) as a Float number. */
@@ -1012,3 +1023,9 @@ Init_Math(void)
     rb_define_module_function(rb_mMath, "gamma", math_gamma, 1);
     rb_define_module_function(rb_mMath, "lgamma", math_lgamma, 1);
 }
+
+void
+Init_Math(void)
+{
+    InitVM(Math);
+}
